'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var vue = require('vue');
var bodyScrollLock = require('body-scroll-lock');

var script$1 = vue.defineComponent({
    name: 'SidePanelCloseButton',
    emits: ['close'],
});

const _hoisted_1 = /*#__PURE__*/vue.createElementVNode("span", { class: "vsp-close__x" }, null, -1 /* HOISTED */);
const _hoisted_2 = [
  _hoisted_1
];

function render$1(_ctx, _cache, $props, $setup, $data, $options) {
  return (vue.openBlock(), vue.createElementBlock("div", {
    class: "vsp-close",
    onClick: _cache[0] || (_cache[0] = $event => (_ctx.$emit('close')))
  }, [..._hoisted_2]))
}

script$1.render = render$1;
script$1.__file = "src/components/SidePanelCloseButton.vue";

var script = vue.defineComponent({
    name: 'VueSidePanel',
    components: {
        SidePanelCloseButton: script$1,
    },
    props: {
        idName: {
            type: String,
            default: 'vsp-container',
        },
        hideCloseBtn: {
            type: Boolean,
            default: false,
        },
        noClose: {
            type: Boolean,
            default: false,
        },
        side: {
            type: String,
            validator: (value) => ['top', 'right', 'bottom', 'left'].includes(value),
            default: 'right',
        },
        rerender: {
            type: Boolean,
            default: false,
        },
        zIndex: {
            type: [Number, String],
            default: 'auto',
        },
        width: {
            type: String,
            default: 'auto',
        },
        height: {
            type: String,
            default: 'auto',
        },
        lockScroll: {
            type: Boolean,
            default: false,
        },
        lockScrollHtml: {
            type: Boolean,
            default: true,
        },
        modelValue: {
            type: Boolean,
            default: false,
            required: true,
        },
        overlayColor: {
            type: String,
            default: 'black',
        },
        overlayOpacity: {
            type: Number,
            default: 0.5,
        },
        overlayDuration: {
            type: Number,
            default: 500,
        },
        panelColor: {
            type: String,
            default: 'white',
        },
        panelDuration: {
            type: Number,
            default: 300,
        },
        transitionName: {
            type: String,
            default: undefined,
        },
        headerClass: {
            type: String,
            default: '',
        },
        bodyClass: {
            type: String,
            default: '',
        },
        footerClass: {
            type: String,
            default: '',
        },
    },
    emits: ['update:modelValue', 'closed', 'opened'],
    setup(props, { emit, attrs }) {
        let teleportContainer = undefined;
        const panel = vue.ref(null);
        const overlay = vue.ref(null);
        const footer = vue.ref(null);
        const header = vue.ref(null);
        const body = vue.ref(null);
        const footerHeight = vue.ref(0);
        const bodyScrollHeight = vue.ref(0);
        const headerHeight = vue.ref(0);
        const panelHeight = vue.ref(0);
        const windowHeight = vue.ref(0);
        const zIndex = vue.ref();
        const isBodyAlreadyLocked = vue.ref(false);
        const calculateRightSize = async () => {
            if (window?.innerHeight > 0)
                windowHeight.value = window.innerHeight;
            footerHeight.value = footer.value ? footer.value.clientHeight : 0;
            headerHeight.value = header.value ? header.value.clientHeight : 0;
            bodyScrollHeight.value = body.value ? body.value.scrollHeight : 0;
            panelHeight.value = panel.value ? panel.value.clientHeight : 0;
        };
        const closePanel = () => emit('update:modelValue', false);
        const lockUnlockBodyScroll = (elem, lock) => {
            if (lock) {
                setTimeout(() => {
                    bodyScrollLock.disableBodyScroll(elem, { reserveScrollBarGap: true });
                    if (props.lockScrollHtml)
                        document.documentElement.style.overflow = 'hidden';
                }, 0);
                return;
            }
            bodyScrollLock.enableBodyScroll(elem);
            if (props.lockScrollHtml)
                document.documentElement.style.removeProperty('overflow');
        };
        const getMaxZIndex = () => Math.max(...Array.from(document.querySelectorAll('body *'), (el) => parseFloat(window.getComputedStyle(el).zIndex)).filter((zIndex) => !Number.isNaN(zIndex)), 0);
        vue.onMounted(() => {
            zIndex.value = props.zIndex === 'auto' ? getMaxZIndex() : props.zIndex;
        });
        vue.onBeforeMount(() => {
            const alreadyCreatedTarget = document.getElementById(props.idName);
            if (!!alreadyCreatedTarget)
                return;
            teleportContainer = document.createElement('div');
            teleportContainer.setAttribute('id', props.idName);
            document.body.appendChild(teleportContainer);
        });
        vue.onBeforeUnmount(() => {
            const { modelValue, lockScroll } = props;
            if (lockScroll && panel.value && modelValue)
                lockUnlockBodyScroll(panel.value, false);
            if (teleportContainer)
                document.body.removeChild(teleportContainer);
            window.removeEventListener('resize', calculateRightSize);
        });
        vue.watch(() => [header.value, footer.value, props.height, props.width, props.side, props.modelValue], () => {
            vue.nextTick(() => calculateRightSize());
        });
        vue.watch(() => [props.modelValue, panel.value], (newP, oldP) => {
            const wasShown = oldP ? oldP[0] : false;
            const [isShown, panelEl] = newP;
            const isOpening = isShown;
            const isClosing = wasShown && !isShown;
            if (!panelEl)
                return;
            if (isOpening)
                isBodyAlreadyLocked.value = !!document.body.style.overflow;
            if (isShown) {
                if (props.lockScroll)
                    lockUnlockBodyScroll(panelEl, true);
                calculateRightSize();
                window.addEventListener('resize', calculateRightSize);
                return;
            }
            if (!props.lockScroll || !isClosing || isBodyAlreadyLocked.value)
                return;
            setTimeout(() => {
                if (panelEl)
                    lockUnlockBodyScroll(panelEl, false);
            }, props.panelDuration);
            window.removeEventListener('resize', calculateRightSize);
        }, { immediate: true });
        const bodyHeight = vue.computed(() => {
            if (!panelHeight.value)
                return;
            const panelMaxHeight = bodyScrollHeight.value + headerHeight.value + footerHeight.value;
            let height = panelHeight.value - headerHeight.value - footerHeight.value;
            if (['top', 'bottom'].includes(props.side) && props.height === 'auto') {
                height =
                    windowHeight.value >= panelMaxHeight
                        ? bodyScrollHeight.value
                        : windowHeight.value - headerHeight.value - footerHeight.value;
            }
            return height;
        });
        const overlayStyles = vue.computed(() => ({
            zIndex: zIndex.value,
            animationDuration: `${props.overlayDuration}ms`,
            '--overlay-opacity': props.overlayOpacity,
            opacity: props.modelValue ? props.overlayOpacity : 0,
            backgroundColor: props.overlayColor,
            pointerEvents: !props.modelValue ? 'none' : 'all',
        }));
        const panelStyles = vue.computed(() => ({
            width: ['left', 'right'].includes(props.side) ? props.width : undefined,
            maxWidth: '100%',
            ...(['top', 'bottom'].includes(props.side)
                ? {
                    // minHeight: props.height,
                    height: props.height,
                    maxHeight: '100%',
                }
                : {}),
            zIndex: zIndex.value,
            backgroundColor: props.panelColor,
            animationDuration: `${props.panelDuration}ms`,
            ...Object.assign({}, attrs.style),
        }));
        return {
            body,
            panel,
            overlay,
            overlayStyles,
            header,
            footer,
            closePanel,
            panelStyles,
            bodyHeight,
        };
    },
});

function render(_ctx, _cache, $props, $setup, $data, $options) {
  const _component_SidePanelCloseButton = vue.resolveComponent("SidePanelCloseButton");

  return (vue.openBlock(), vue.createBlock(vue.Teleport, {
    to: `#${_ctx.idName}`
  }, [
    vue.createElementVNode("div", {
      class: vue.normalizeClass(["vsp-wrapper", [_ctx.modelValue && 'vsp-wrapper--active']])
    }, [
      vue.createVNode(vue.Transition, {
        name: "overlay",
        persisted: ""
      }, {
        default: vue.withCtx(() => [
          vue.withDirectives(vue.createElementVNode("div", {
            ref: "overlay",
            class: "vsp-overlay",
            style: vue.normalizeStyle(_ctx.overlayStyles),
            onClick: _cache[0] || (_cache[0] = () => (_ctx.noClose ? undefined : _ctx.closePanel()))
          }, null, 4 /* STYLE */), [
            [vue.vShow, _ctx.modelValue]
          ])
        ]),
        _: 1 /* STABLE */
      }),
      vue.createVNode(vue.Transition, {
        name: _ctx.transitionName || `slide-${_ctx.side}`,
        onAfterEnter: _cache[1] || (_cache[1] = $event => (_ctx.$emit('opened'))),
        onAfterLeave: _cache[2] || (_cache[2] = $event => (_ctx.$emit('closed')))
      }, {
        default: vue.withCtx(() => [
          (_ctx.rerender ? _ctx.modelValue : true)
            ? vue.withDirectives((vue.openBlock(), vue.createElementBlock("div", {
                key: 0,
                ref: "panel",
                class: vue.normalizeClass(["vsp", [`vsp--${_ctx.side}-side`, _ctx.$attrs.class]]),
                style: vue.normalizeStyle(_ctx.panelStyles)
              }, [
                (!!_ctx.$slots.header)
                  ? (vue.openBlock(), vue.createElementBlock("div", {
                      key: 0,
                      ref: "header",
                      class: vue.normalizeClass([_ctx.headerClass, 'vsp__header'])
                    }, [
                      vue.renderSlot(_ctx.$slots, "header", { close: _ctx.closePanel })
                    ], 2 /* CLASS */))
                  : vue.createCommentVNode("v-if", true),
                vue.createElementVNode("div", {
                  ref: "body",
                  class: vue.normalizeClass([_ctx.bodyClass, 'vsp__body']),
                  style: vue.normalizeStyle({ height: `${_ctx.bodyHeight}px` })
                }, [
                  vue.renderSlot(_ctx.$slots, "default", { close: _ctx.closePanel }),
                  (!_ctx.hideCloseBtn)
                    ? (vue.openBlock(), vue.createBlock(_component_SidePanelCloseButton, {
                        key: 0,
                        onClose: _ctx.closePanel
                      }, null, 8 /* PROPS */, ["onClose"]))
                    : vue.createCommentVNode("v-if", true)
                ], 6 /* CLASS, STYLE */),
                (!!_ctx.$slots.footer)
                  ? (vue.openBlock(), vue.createElementBlock("div", {
                      key: 1,
                      ref: "footer",
                      class: vue.normalizeClass([_ctx.footerClass, 'vsp__footer'])
                    }, [
                      vue.renderSlot(_ctx.$slots, "footer")
                    ], 2 /* CLASS */))
                  : vue.createCommentVNode("v-if", true)
              ], 6 /* CLASS, STYLE */)), [
                [vue.vShow, _ctx.rerender ? true : _ctx.modelValue]
              ])
            : vue.createCommentVNode("v-if", true)
        ]),
        _: 3 /* FORWARDED */
      }, 8 /* PROPS */, ["name"])
    ], 2 /* CLASS */)
  ], 8 /* PROPS */, ["to"]))
}

script.render = render;
script.__file = "src/components/SidePanel.vue";

var VueSidePanelPlugin = {
    install(app) {
        app.component('VueSidePanel', script);
    },
};

exports.VueSidePanel = script;
exports["default"] = VueSidePanelPlugin;
//# sourceMappingURL=vue3-side-panel.js.map
